/*
 * This file is part of the coreboot project.
 *
 * It is based on the include/linux/pci_ids.h file from the Linux kernel.
 *
 * Note: we maintain our own short list of PCI vendor IDs and device IDs
 * for coreboot, as we only need very few of the entries.
 * The 'Device classes and subclasses' section is copied from the Linux file.
 */

#ifndef DEVICE_PCI_IDS_H
#define DEVICE_PCI_IDS_H

/* Device classes and subclasses. */

#define PCI_CLASS_NOT_DEFINED			0x0000
#define PCI_CLASS_NOT_DEFINED_VGA		0x0001

#define PCI_BASE_CLASS_STORAGE			0x01
#define PCI_CLASS_STORAGE_SCSI			0x0100
#define PCI_CLASS_STORAGE_IDE			0x0101
#define PCI_CLASS_STORAGE_FLOPPY		0x0102
#define PCI_CLASS_STORAGE_IPI			0x0103
#define PCI_CLASS_STORAGE_RAID			0x0104
#define PCI_CLASS_STORAGE_SATA			0x0106
#define PCI_CLASS_STORAGE_SATA_AHCI		0x010601
#define PCI_CLASS_STORAGE_SAS			0x0107
#define PCI_CLASS_STORAGE_OTHER			0x0180

#define PCI_BASE_CLASS_NETWORK			0x02
#define PCI_CLASS_NETWORK_ETHERNET		0x0200
#define PCI_CLASS_NETWORK_TOKEN_RING		0x0201
#define PCI_CLASS_NETWORK_FDDI			0x0202
#define PCI_CLASS_NETWORK_ATM			0x0203
#define PCI_CLASS_NETWORK_OTHER			0x0280

#define PCI_BASE_CLASS_DISPLAY			0x03
#define PCI_CLASS_DISPLAY_VGA			0x0300
#define PCI_CLASS_DISPLAY_XGA			0x0301
#define PCI_CLASS_DISPLAY_3D			0x0302
#define PCI_CLASS_DISPLAY_OTHER			0x0380

#define PCI_BASE_CLASS_MULTIMEDIA		0x04
#define PCI_CLASS_MULTIMEDIA_VIDEO		0x0400
#define PCI_CLASS_MULTIMEDIA_AUDIO		0x0401
#define PCI_CLASS_MULTIMEDIA_PHONE		0x0402
#define PCI_CLASS_MULTIMEDIA_OTHER		0x0480

#define PCI_BASE_CLASS_MEMORY			0x05
#define PCI_CLASS_MEMORY_RAM			0x0500
#define PCI_CLASS_MEMORY_FLASH			0x0501
#define PCI_CLASS_MEMORY_OTHER			0x0580

#define PCI_BASE_CLASS_BRIDGE			0x06
#define PCI_CLASS_BRIDGE_HOST			0x0600
#define PCI_CLASS_BRIDGE_ISA			0x0601
#define PCI_CLASS_BRIDGE_EISA			0x0602
#define PCI_CLASS_BRIDGE_MC			0x0603
#define PCI_CLASS_BRIDGE_PCI			0x0604
#define PCI_CLASS_BRIDGE_PCMCIA			0x0605
#define PCI_CLASS_BRIDGE_NUBUS			0x0606
#define PCI_CLASS_BRIDGE_CARDBUS		0x0607
#define PCI_CLASS_BRIDGE_RACEWAY		0x0608
#define PCI_CLASS_BRIDGE_OTHER			0x0680

#define PCI_BASE_CLASS_COMMUNICATION		0x07
#define PCI_CLASS_COMMUNICATION_SERIAL		0x0700
#define PCI_CLASS_COMMUNICATION_PARALLEL	0x0701
#define PCI_CLASS_COMMUNICATION_MULTISERIAL	0x0702
#define PCI_CLASS_COMMUNICATION_MODEM		0x0703
#define PCI_CLASS_COMMUNICATION_OTHER		0x0780

#define PCI_BASE_CLASS_SYSTEM			0x08
#define PCI_CLASS_SYSTEM_PIC			0x0800
#define PCI_CLASS_SYSTEM_PIC_IOAPIC		0x080010
#define PCI_CLASS_SYSTEM_PIC_IOXAPIC		0x080020
#define PCI_CLASS_SYSTEM_DMA			0x0801
#define PCI_CLASS_SYSTEM_TIMER			0x0802
#define PCI_CLASS_SYSTEM_RTC			0x0803
#define PCI_CLASS_SYSTEM_PCI_HOTPLUG		0x0804
#define PCI_CLASS_SYSTEM_SDHCI			0x0805
#define PCI_CLASS_SYSTEM_OTHER			0x0880

#define PCI_BASE_CLASS_INPUT			0x09
#define PCI_CLASS_INPUT_KEYBOARD		0x0900
#define PCI_CLASS_INPUT_PEN			0x0901
#define PCI_CLASS_INPUT_MOUSE			0x0902
#define PCI_CLASS_INPUT_SCANNER			0x0903
#define PCI_CLASS_INPUT_GAMEPORT		0x0904
#define PCI_CLASS_INPUT_OTHER			0x0980

#define PCI_BASE_CLASS_DOCKING			0x0a
#define PCI_CLASS_DOCKING_GENERIC		0x0a00
#define PCI_CLASS_DOCKING_OTHER			0x0a80

#define PCI_BASE_CLASS_PROCESSOR		0x0b
#define PCI_CLASS_PROCESSOR_386			0x0b00
#define PCI_CLASS_PROCESSOR_486			0x0b01
#define PCI_CLASS_PROCESSOR_PENTIUM		0x0b02
#define PCI_CLASS_PROCESSOR_ALPHA		0x0b10
#define PCI_CLASS_PROCESSOR_POWERPC		0x0b20
#define PCI_CLASS_PROCESSOR_MIPS		0x0b30
#define PCI_CLASS_PROCESSOR_CO			0x0b40

#define PCI_BASE_CLASS_SERIAL			0x0c
#define PCI_CLASS_SERIAL_FIREWIRE		0x0c00
#define PCI_CLASS_SERIAL_FIREWIRE_OHCI		0x0c0010
#define PCI_CLASS_SERIAL_ACCESS			0x0c01
#define PCI_CLASS_SERIAL_SSA			0x0c02
#define PCI_CLASS_SERIAL_USB			0x0c03
#define PCI_CLASS_SERIAL_USB_UHCI		0x0c0300
#define PCI_CLASS_SERIAL_USB_OHCI		0x0c0310
#define PCI_CLASS_SERIAL_USB_EHCI		0x0c0320
#define PCI_CLASS_SERIAL_FIBER			0x0c04
#define PCI_CLASS_SERIAL_SMBUS			0x0c05

#define PCI_BASE_CLASS_WIRELESS			0x0d
#define PCI_CLASS_WIRELESS_RF_CONTROLLER	0x0d10
#define PCI_CLASS_WIRELESS_WHCI			0x0d1010

#define PCI_BASE_CLASS_INTELLIGENT		0x0e
#define PCI_CLASS_INTELLIGENT_I2O		0x0e00

#define PCI_BASE_CLASS_SATELLITE		0x0f
#define PCI_CLASS_SATELLITE_TV			0x0f00
#define PCI_CLASS_SATELLITE_AUDIO		0x0f01
#define PCI_CLASS_SATELLITE_VOICE		0x0f03
#define PCI_CLASS_SATELLITE_DATA		0x0f04

#define PCI_BASE_CLASS_CRYPT			0x10
#define PCI_CLASS_CRYPT_NETWORK			0x1000
#define PCI_CLASS_CRYPT_ENTERTAINMENT		0x1001
#define PCI_CLASS_CRYPT_OTHER			0x1080

#define PCI_BASE_CLASS_SIGNAL_PROCESSING	0x11
#define PCI_CLASS_SP_DPIO			0x1100
#define PCI_CLASS_SP_OTHER			0x1180

#define PCI_CLASS_OTHERS			0xff

/* Vendors and devices. Sort key: vendor first, device next. */

#define PCI_VENDOR_ID_AMD			0x1022
#define PCI_VENDOR_ID_AMD		0x1022
#define PCI_DEVICE_ID_AMD_LANCE		0x2000
#define PCI_DEVICE_ID_AMD_LANCE_HOME	0x2001
#define PCI_DEVICE_ID_AMD_LX		0x1054
#define PCI_DEVICE_ID_AMD_SCSI		0x2020
#define PCI_DEVICE_ID_AMD_FE_GATE_7006	0x7006
#define PCI_DEVICE_ID_AMD_FE_GATE_7007	0x7007
#define PCI_DEVICE_ID_AMD_FE_GATE_700C	0x700C
#define PCI_DEVIDE_ID_AMD_FE_GATE_700D	0x700D
#define PCI_DEVICE_ID_AMD_FE_GATE_700E	0x700E
#define PCI_DEVICE_ID_AMD_FE_GATE_700F	0x700F
#define PCI_DEVICE_ID_AMD_COBRA_7400	0x7400
#define PCI_DEVICE_ID_AMD_COBRA_7401	0x7401
#define PCI_DEVICE_ID_AMD_COBRA_7403	0x7403
#define PCI_DEVICE_ID_AMD_COBRA_7404	0x7404
#define PCI_DEVICE_ID_AMD_VIPER_7408	0x7408
#define PCI_DEVICE_ID_AMD_VIPER_7409	0x7409
#define PCI_DEVICE_ID_AMD_VIPER_740B	0x740B
#define PCI_DEVICE_ID_AMD_VIPER_740C	0x740C
#define PCI_DEVICE_ID_AMD_VIPER_7410	0x7410
#define PCI_DEVICE_ID_AMD_VIPER_7411	0x7411
#define PCI_DEVICE_ID_AMD_SB768_7440  0x7440
#define PCI_DEVICE_ID_AMD_SB768_7441  0x7441
#define PCI_DEVICE_ID_AMD_SB768_7443  0x7443
#define PCI_DEVICE_ID_AMD_SB768_7448  0x7448
#define PCI_DEVICE_ID_AMD_SB768_7449  0x7449
#define PCI_DEVICE_ID_AMD_VIPER_7413	0x7413
#define PCI_DEVICE_ID_AMD_VIPER_7414	0x7414
#define PCI_DEVICE_ID_AMD_VIPER_7440	0x7440
#define PCI_DEVICE_ID_AMD_VIPER_7441	0x7441
#define PCI_DEVICE_ID_AMD_VIPER_7443	0x7443
#define PCI_DEVICE_ID_AMD_VIPER_7448	0x7448
#define PCI_DEVICE_ID_AMD_VIPER_7449	0x7449
#define PCI_DEVICE_ID_AMD_8151_SYSCTRL  0x7454
#define PCI_DEVICE_ID_AMD_8151_AGP      0x7455
#define PCI_DEVICE_ID_AMD_8111_PCI      0x7460
#define PCI_DEVICE_ID_AMD_8111_USB      0x7464
#define PCI_DEVICE_ID_AMD_8111_ISA      0x7468
#define PCI_DEVICE_ID_AMD_8111_IDE      0x7469
#define PCI_DEVICE_ID_AMD_8111_SMB      0x746a
#define PCI_DEVICE_ID_AMD_8111_ACPI     0x746b
#define PCI_DEVICE_ID_AMD_8111_NIC    	0x7462
#define PCI_DEVICE_ID_AMD_8111_USB2     0x7463
#define PCI_DEVICE_ID_AMD_8131_PCIX	0x7450
#define PCI_DEVICE_ID_AMD_8131_IOAPIC   0x7451
#define PCI_DEVICE_ID_AMD_8132_PCIX     0x7458
#define PCI_DEVICE_ID_AMD_8132_IOAPIC   0x7459
#define PCI_DEVICE_ID_AMD_AES		0x2082
#define PCI_DEVICE_ID_AMD_761_0         0x700E
#define PCI_DEVICE_ID_AMD_761_1         0x700F
#define PCI_DEVICE_ID_AMD_VIPER_7412    0x7412
#define PCI_DEVICE_ID_AMD_LXBRIDGE		0x2080
#define PCI_DEVICE_ID_AMD_CS5536_ISA		0x2090
#define PCI_DEVICE_ID_AMD_CS5536_FLASH		0x2091
#define PCI_DEVICE_ID_AMD_CS5536_A0_IDE		0x2092
#define PCI_DEVICE_ID_AMD_CS5536_AUDIO		0x2093
#define PCI_DEVICE_ID_AMD_CS5536_OHCI		0x2094
#define PCI_DEVICE_ID_AMD_CS5536_EHCI		0x2095
#define PCI_DEVICE_ID_AMD_CS5536_UDC		0x2096
#define PCI_DEVICE_ID_AMD_CS5536_OTG		0x2097
#define PCI_DEVICE_ID_AMD_CS5536_B0_IDE		0x209A

#define PCI_VENDOR_ID_CIRRUS			0x1013
#define PCI_DEVICE_ID_CIRRUS_5446		0x00b8	/* Used by QEMU */

#define PCI_VENDOR_ID_NVIDIA                    0x10de
#define PCI_DEVICE_ID_NVIDIA_MCP55_LPC          0x0360
#define PCI_DEVICE_ID_NVIDIA_MCP55_SLAVE        0x0361
#define PCI_DEVICE_ID_NVIDIA_MCP55_LPC_2        0x0362
#define PCI_DEVICE_ID_NVIDIA_MCP55_LPC_3        0x0363
#define PCI_DEVICE_ID_NVIDIA_MCP55_LPC_4        0x0364
#define PCI_DEVICE_ID_NVIDIA_MCP55_LPC_5        0x0365
#define PCI_DEVICE_ID_NVIDIA_MCP55_LPC_6        0x0366
#define PCI_DEVICE_ID_NVIDIA_MCP55_PRO          0x0367
#define PCI_DEVICE_ID_NVIDIA_MCP55_SM2          0x0368
#define PCI_DEVICE_ID_NVIDIA_MCP55_IDE          0x036E
#define PCI_DEVICE_ID_NVIDIA_MCP55_SATA0        0x037E
#define PCI_DEVICE_ID_NVIDIA_MCP55_SATA1        0x037F
#define PCI_DEVICE_ID_NVIDIA_MCP55_NIC          0x0372
#define PCI_DEVICE_ID_NVIDIA_MCP55_NIC_BRIDGE   0x0373
#define PCI_DEVICE_ID_NVIDIA_MCP55_AZA          0x0371
#define PCI_DEVICE_ID_NVIDIA_MCP55_USB          0x036C
#define PCI_DEVICE_ID_NVIDIA_MCP55_USB2         0x036D
#define PCI_DEVICE_ID_NVIDIA_MCP55_PCI          0x0370
#define PCI_DEVICE_ID_NVIDIA_MCP55_PCIE_B_C     0x0374
#define PCI_DEVICE_ID_NVIDIA_MCP55_PCIE_E       0x0375
#define PCI_DEVICE_ID_NVIDIA_MCP55_PCIE_A       0x0376
#define PCI_DEVICE_ID_NVIDIA_MCP55_PCIE_F       0x0377
#define PCI_DEVICE_ID_NVIDIA_MCP55_PCIE_D       0x0378
#define PCI_DEVICE_ID_NVIDIA_MCP55_HT           0x0369
#define PCI_DEVICE_ID_NVIDIA_MCP55_TRIM         0x036A
#define PCI_DEVICE_ID_NVIDIA_MCP55_PMU          0x036B

#endif /* DEVICE_PCI_IDS_H */
